package com.huigou.topsun.sap.nonProdApply.application.impl;


import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.huigou.topsun.base.bsnMessage.appliction.BsnMessageApplication;
import com.huigou.topsun.base.bsnMessage.domain.*;
import com.huigou.topsun.sap.nonProdApply.application.NonProdApplyDeptApplication;

import com.huigou.topsun.sap.nonProdApply.domain.SapNonProdApplyDept;
import com.huigou.topsun.sap.nonProdApply.repository.NonProdApplyDeptRepository;
import com.huigou.uasp.bmp.common.application.BaseApplication;

import com.huigou.util.SDO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class NonProdApplyDeptApplicationImpl extends BaseApplication implements NonProdApplyDeptApplication {

    @Autowired
    NonProdApplyDeptRepository nonProdApplyDeptRepository;
    @Autowired
    BsnMessageApplication bsnMessageApplication;

    @Override
    public Map<String,Object> saveSapNonProdApplyDept(SDO sdo){
        Map<String,Object> map = new HashMap<>();
        SapNonProdApplyDept nonProdApplyDept = sdo.toObject(SapNonProdApplyDept.class);
        nonProdApplyDept = nonProdApplyDeptRepository.save(nonProdApplyDept);

        //获取分发的部门与人员信息
        ObjectMapper objectMapper = new ObjectMapper();
        String deptStaffJSON = sdo.getString("deptStaff");
        List<Map<String, String>> deptStaffList = null;
        try {
            deptStaffList = objectMapper.readValue(deptStaffJSON, new TypeReference<List<Map<String, String>>>() {});
        } catch (JsonProcessingException e) {
            throw new RuntimeException("JSON转换失败："+e);
        }


        //封装分发数据
        BsnMessageSponsor bsnMessageSponsor = new BsnMessageSponsor();
        bsnMessageSponsor.setTaskSponsor(this.getOperator().getUserId());
        bsnMessageSponsor.setTitle("非产品/非本成品类物料");
        bsnMessageSponsor.setExecutorUrl("sapNonProdApplyNext/fillInNonProdApplyDept.do");
        bsnMessageSponsor.setBusinessId(nonProdApplyDept.getApplyDeptId());
        bsnMessageSponsor.setCreateDate(new Date());
        bsnMessageSponsor.setStatus(0);

        for (Map<String, String> stringMap : deptStaffList) {
            BsnMessageExecute messageExecute = new BsnMessageExecute();
            String userCode = stringMap.get("userCode");
            String deptName = stringMap.get("deptName");
            String deptCode = stringMap.get("deptCode");
            String[] split = userCode.split("@");
            messageExecute.setTaskExecute(split[0]);
            messageExecute.setBusinessCode(deptCode);
            messageExecute.setActive(ActiveStatus.SAVED.getValue());
            messageExecute.setUpdateDate(new Date());
            messageExecute.setTaskDescribe(deptName+"部门数据维护");
            bsnMessageSponsor.addExecute(messageExecute);
        }
        //保存任务
        bsnMessageApplication.keepTasks(bsnMessageSponsor);
        map.put("applyDeptId",nonProdApplyDept.getApplyDeptId());
        return map;
    }

    /**
     * 发布任务
     * @param sdo
     */
    @Override
    public void distributeSapNonProdApplyDept(SDO sdo) {
        //将状态为保存未分发的全部修改为分发未完成
        Map<String, Object> map = this.saveSapNonProdApplyDept(sdo);
        bsnMessageApplication.publishTasks((String) map.get("applyDeptId"));
    }

    @Override
    public SapNonProdApplyDept getNonProdApplyDeptById(String id) {
        return nonProdApplyDeptRepository.findOne(id);
    }


    @Override
    public void rejectNonProdApplyDept(SDO sdo) {
        String meId = sdo.getString("meId");
        Map<String,Object> map = new HashMap<>();
        String auditDept = sdo.getString("auditDept");
        List<String> deptList = Arrays.asList(auditDept.split(","));
        for (String key : deptList) {
            map.put(key,sdo.getString(key));
        }
        bsnMessageApplication.rejectTasksByMeId(map,meId);
    }
}
